<?php

namespace Drupal\Tests\external_entities\Functional;

use Drupal\external_entities\FieldMapper\FieldMapperInterface;
use Drupal\Tests\TestFileCreationTrait;

/**
 * Tests file storage client for external entity.
 *
 * @group ExternalEntities
 */
class FileClientFunctionalTest extends ExternalEntitiesBrowserTestBase {

  use TestFileCreationTrait;

  /**
   * File system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * A user with administration permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $account;

  /**
   * The entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $storage;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->fileSystem = $this->container->get('file_system');
    $file_path = $this->generateFile('test', 21, 2, 'text');
    $filename = $this->fileSystem->basename($file_path);
    $directory = $this->fileSystem->dirname($file_path);
    $this->assertFileExists($file_path);

    // Storage client.
    $this->storage = $this->container->get('entity_type.manager')->getStorage('external_entity_type');

    /** @var \Drupal\external_entities\Entity\ExternalEntityType $type */
    $type = $this->container->get('entity_type.manager')->getStorage('external_entity_type')->create([
      'id' => 'test_file_xntt',
      'label' => 'Simple external entity',
      'label_plural' => 'Simple external entities',
      'description' => '',
      'read_only' => FALSE,
      'generate_aliases' => FALSE,
      'persistent_cache_max_age' => 0,
    ]);

    // Set Field mapper.
    $type->setFieldMapperId('simple');
    $type->setFieldMapperConfig([
      'field_mappings' => [
        'id' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'id',
          ],
        ],
      ],
    ]);

    // Sets client.
    $type->setStorageClientId('files');
    $type->setStorageClientConfig([
      'root' => $directory,
      'structure' => $filename,
      'matching_only' => FALSE,
      'multi_records' => FALSE,
      'performances' => [
        'use_index' => FALSE,
        'index_file' => '',
      ],
      'data_fields' => [
        'md5_checksum' => FALSE,
        'sha1_checksum' => FALSE,
        'field_list' => [],
      ],
    ],
    0);

    // Sets aggregator.
    $type->setDataAggregatorId('group');
    $type->setDataAggregatorConfig([
      'global_config' => [],
      'storage_client_config' => [
        [
          'groups' => [],
          'group_prefix_strip' => FALSE,
          'id_field' => '',
          'merge' => 'keep',
          'merge_as_member' => '',
          'merge_join' => '',
          'readonly' => FALSE,
          'client_number' => 0,
        ],
        [
          'groups' => [],
          'group_prefix_strip' => FALSE,
          'id_field' => '',
          'merge' => 'keep',
          'merge_as_member' => '',
          'merge_join' => '',
          'readonly' => FALSE,
          'client_number' => 0,
        ],
      ],
    ]);
    $type->save();

    // Add fields.
    $this->createField('test_file_xntt', 'field_path', 'string');
    $this->createField('test_file_xntt', 'field_dirname', 'string');
    $this->createField('test_file_xntt', 'field_basename', 'string');
    $this->createField('test_file_xntt', 'field_filename', 'string');
    $this->createField('test_file_xntt', 'field_extension', 'string');
    $this->createField('test_file_xntt', 'field_mode', 'string');
    $this->createField('test_file_xntt', 'field_size', 'integer');
    $this->createField('test_file_xntt', 'field_mtime', 'timestamp');
    $this->createField('test_file_xntt', 'field_md5', 'string');
    $this->createField('test_file_xntt', 'field_sha1', 'string');

    $type->setFieldMapperConfig([
      'field_mappings' => [
        'id' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'id',
          ],
        ],
        'uuid' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'id',
          ],
        ],
        'title' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'filename',
          ],
        ],
        'field_path' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'path',
          ],
        ],
        'field_dirname' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'dirname',
          ],
        ],
        'field_basename' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'basename',
          ],
        ],
        'field_filename' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'filename',
          ],
        ],
        'field_extension' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'extension',
          ],
        ],
        'field_mode' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'mode',
          ],
        ],
        'field_size' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'size',
          ],
        ],
        'field_mtime' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'mtime',
          ],
        ],
        'field_md5' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'md5',
          ],
        ],
        'field_sha1' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'sha1',
          ],
        ],
      ],
    ]);
    $type->save();

    // Create the user with all needed permissions.
    $this->account = $this->drupalCreateUser([
      'administer external entity types',
      'view test_file_xntt external entity',
      'view test_file_xntt external entity collection',
      'access site reports',
    ]);
    $this->drupalLogin($this->account);
  }

  /**
   * Tests file storage client global features.
   */
  public function testFileClient() {
    /** @var \Drupal\Tests\WebAssert $assert */
    $assert = $this->assertSession();

    // Entity list check.
    $this->drupalGet('test-file-xntt');
    $assert->pageTextContains('test.txt');

    // First file test.
    $this->drupalGet('test-file-xntt/test.txt');
    $assert->pageTextContains('test.txt');
    $assert->pageTextContains('public:///test.txt');
    $assert->pageTextContains('public:');
    $assert->pageTextContains('33204');
    $assert->pageTextContains('42');
    $assert->pageTextContains('field_mtime');

    $this->drupalGet('test-file-xntt/test.txt/edit');
    $assert->pageTextContains('test.txt');

    // phpcs:disable
    // $this->drupalGet('admin/reports/dblog');
    // phpcs:enable

    // @todo Implement deeper tests.
    $this->markTestIncomplete('Needs more tests.');
  }

}
