<?php

namespace Drupal\Tests\external_entities\Functional;

use Drupal\Component\Serialization\Json;
use Drupal\external_entities\FieldMapper\FieldMapperInterface;

/**
 * Test external entities query system.
 *
 * @group ExternalEntities
 */
class QueryTest extends ExternalEntitiesBrowserTestBase {


  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'external_entities',
    'system',
    'field',
    'text',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    global $base_url;

    $this->entity_type_manager = $this->container->get('entity_type.manager');
    $this->storage = $this->entity_type_manager->getStorage('external_entity_type');

    // Setup datasets.
    $xntt_json_controller = $this
      ->container
      ->get('controller_resolver')
      ->getControllerFromDefinition(
        '\Drupal\external_entities_test\Controller\ExternalEntitiesJsonController::listItems'
      )[0];

    $this->rawData = [
      1001 => [
        'id' => 1001,
        'title' => 'Instance 1',
        'data_field_a' => 'toto',
        'data_field_b' => 42,
        'data_field_c' => ['abc', 'def', 'ghi'],
        'data_field_d' => [
          'sub_field_d1' => 'A',
          'sub_field_d2' => 'B',
        ],
      ],
      1002 => [
        'id' => 1002,
        'title' => 'Instance 2',
        'data_field_a' => 'tata',
        'data_field_b' => 501,
        'data_field_c' => ['abc', 'def'],
        'data_field_d' => [
          'sub_field_d1' => 'A',
          'sub_field_d2' => 'B',
        ],
      ],
      1003 => [
        'id' => 1003,
        'title' => 'Instance 3',
        'data_field_a' => 'tete',
        'data_field_b' => 42,
        'data_field_c' => ['abc'],
        'data_field_d' => [
          'sub_field_d1' => 'A',
          'sub_field_d2' => 'B',
        ],
      ],
      1004 => [
        'id' => 1004,
        'title' => 'Instance 4',
        'data_field_a' => 'titi',
        'data_field_b' => 806,
        'data_field_c' => ['def', 'ghi'],
        'data_field_d' => [
          'sub_field_d1' => 'C',
          'sub_field_d2' => 'C',
        ],
      ],
      1005 => [
        'id' => 1005,
        'title' => 'Instance 5',
        'data_field_a' => 'tutu',
        'data_field_b' => 77,
        'data_field_c' => ['ghi'],
        'data_field_d' => [
          'sub_field_d1' => 'D',
          'sub_field_d2' => 'E',
        ],
      ],
      1006 => [
        'id' => 1006,
        'title' => 'Instance 6',
        'data_field_a' => 'tota',
        'data_field_b' => 64,
        'data_field_c' => ['def'],
        'data_field_d' => [
          'sub_field_d1' => 'B',
          'sub_field_d2' => 'A',
        ],
      ],
      1007 => [
        'id' => 1007,
        'title' => 'Instance 7',
        'data_field_a' => 'tote',
        'data_field_b' => 51,
        'data_field_c' => ['xyz'],
        'data_field_d' => [
          'sub_field_d1' => '',
          'sub_field_d2' => NULL,
        ],
      ],
      1008 => [
        'id' => 1008,
        'title' => 'Instance 8',
        'data_field_a' => 'toti',
        'data_field_b' => 42,
        'data_field_c' => ['ghi', 'def', 'abc'],
        'data_field_d' => [
          'sub_field_d1' => 'AX',
          'sub_field_d2' => 'BX',
        ],
      ],
      1009 => [
        'id' => 1009,
        'title' => 'Instance 9',
        'data_field_a' => 'totu',
        'data_field_b' => 34,
        'data_field_c' => ['abc', 'def', 'ghi', 'xyz', 'abc', 'def'],
        'data_field_d' => [
          'sub_field_d1' => 'a',
          'sub_field_d2' => 'b',
        ],
      ],
      1010 => [
        'id' => 1010,
        'title' => 'Instance 10',
        'data_field_a' => 'foo',
        'data_field_b' => 21,
        'data_field_c' => [],
        'data_field_d' => [
          'sub_field_d1' => 'X',
          'sub_field_d2' => 'X',
        ],
      ],
      1011 => [
        'id' => 1011,
        'title' => 'Instance 11',
        'data_field_a' => 'bar',
        'data_field_b' => 16,
        'data_field_c' => [NULL],
        'data_field_d' => [
          'sub_field_d1' => 'A',
          'sub_field_d2' => 'B',
        ],
      ],
      1012 => [
        'id' => 1012,
        'title' => 'Instance 12',
        'data_field_a' => 'baz',
        'data_field_b' => 806,
        'data_field_c' => ['abcdefghi'],
        'data_field_d' => [
          'sub_field_d1' => 'C',
          'sub_field_d2' => 'D',
        ],
      ],
    ];

    $xntt_json_controller->setRawData('simple', $this->rawData);

    $type = $this->storage->create([
      'id' => 'simple_external_entity',
      'label' => 'Simple external entity',
      'label_plural' => 'Simple external entities',
      'description' => '',
      'read_only' => TRUE,
      'generate_aliases' => FALSE,
      'persistent_cache_max_age' => 0,
    ]);

    // Explicitely sets client 0.
    $type->setStorageClientId('rest', 0);
    $type->setStorageClientConfig([
      'endpoint' => $base_url . '/external-entities-test/simple',
      'endpoint_options' => [
        'single' => '',
        'count' => count($this->rawData),
        'count_mode' => NULL,
        'cache' => FALSE,
        'limit_qcount' => 0,
        'limit_qtime' => 0,
      ],
      'response_format' => 'json',
      'data_path' => [
        'list' => '',
        'single' => '',
        'keyed_by_id' => FALSE,
        'count' => '',
      ],
      'pager' => [
        'default_limit' => count($this->rawData),
        'page_parameter' => '',
        'page_parameter_type' => 'pagenum',
        'page_start_one' => FALSE,
        'page_size_parameter' => '',
        'page_size_parameter_type' => 'pagesize',
      ],
      'api_key' => [
        'type' => 'none',
        'header_name' => '',
        'key' => '',
      ],
      'http' => [
        'headers' => '',
      ],
      'parameters' => [
        'list' => [],
        'list_param_mode' => 'query',
        'single' => [],
        'single_param_mode' => 'query',
      ],
    ],
    0);

    // Sets aggregator.
    $type->setDataAggregatorId('group');
    $type->setDataAggregatorConfig([
      'global_config' => [],
      'storage_client_config' => [
        [
          'groups' => [],
          'group_prefix_strip' => FALSE,
          'id_field' => '',
          'merge' => 'keep',
          'merge_as_member' => '',
          'merge_join' => '',
          'readonly' => FALSE,
          'client_number' => 0,
        ],
        [
          'groups' => [],
          'group_prefix_strip' => FALSE,
          'id_field' => '',
          'merge' => 'keep',
          'merge_as_member' => '',
          'merge_join' => '',
          'readonly' => FALSE,
          'client_number' => 0,
        ],
      ],
    ]);

    // Set field mapper.
    $type->setFieldMapperId('simple');
    $type->save();

    // Add fields.
    $this->createField('simple_external_entity', 'field_alpha', 'string');
    $this->createField('simple_external_entity', 'field_beta', 'integer');
    $this->createField('simple_external_entity', 'field_gama', 'string');

    $type->setFieldMapperConfig([
      'field_mappings' => [
        'id' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'id',
          ],
        ],
        'uuid' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'id',
          ],
        ],
        'title' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_DIRECT,
            'mapping' => 'title',
          ],
        ],
        'field_alpha' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_SIMPLE,
            'mapping' => 'data_field_a',
          ],
        ],
        'field_beta' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_SIMPLE,
            'mapping' => 'data_field_b',
          ],
        ],
        'field_gama' => [
          'value' => [
            'type' => FieldMapperInterface::MAPPING_SIMPLE,
            'mapping' => 'data_field_c',
          ],
        ],
      ],
    ]);
    $type->save();
  }

  /**
   * Test external entity query system.
   */
  public function testQuery() {

    // Check "simple test endpoint".
    $simple_json = Json::decode($this->drupalGet('external-entities-test/simple'));
    $this
      ->assertSession()
      ->statusCodeEquals(200);
    $this
      ->assertCount(count($this->rawData), $simple_json);
    $this
      ->assertSession()
      ->responseHeaderEquals('Content-Type', 'application/json');

    // Basic query, get all.
    $query = $this->entity_type_manager->getStorage('simple_external_entity')->getQuery();
    $result = $query->accessCheck(FALSE)->execute();
    $this->assertEquals(
      [
        1001 => 1001,
        1002 => 1002,
        1003 => 1003,
        1004 => 1004,
        1005 => 1005,
        1006 => 1006,
        1007 => 1007,
        1008 => 1008,
        1009 => 1009,
        1010 => 1010,
        1011 => 1011,
        1012 => 1012,
      ],
      $result,
      'basic query'
    );

    // Basic query, get the first 3.
    $query = $this->entity_type_manager->getStorage('simple_external_entity')->getQuery();
    $result = $query->pager(3)->accessCheck(FALSE)->execute();
    $this->assertEquals(
      [
        1001 => 1001,
        1002 => 1002,
        1003 => 1003,
      ],
      $result,
      'basic query with pager(3)'
    );

    // Basic query with title = Instance 4.
    $query = $this->entity_type_manager->getStorage('simple_external_entity')->getQuery();
    $result = $query->condition('title', 'Instance 4')->accessCheck(FALSE)->execute();
    $this->assertEquals(
      [
        1004 => 1004,
      ],
      $result,
      'basic query with title = Instance 4'
    );

    // Basic query with field_alpha ending by "ta".
    $query = $this->entity_type_manager->getStorage('simple_external_entity')->getQuery();
    $result = $query->condition('field_alpha', 'ta', 'ENDS_WITH')->accessCheck(FALSE)->execute();
    $this->assertEquals(
      [
        1002 => 1002,
        1006 => 1006,
      ],
      $result,
      'basic query with field_alpha ending by ta'
    );

    // Query with field_alpha ending by "ta" or starting with "te".
    $query = $this->entity_type_manager->getStorage('simple_external_entity')->getQuery();
    $or = $query->orConditionGroup()->condition('field_alpha', 'ta', 'ENDS_WITH')->condition('field_alpha', 'te', 'STARTS_WITH');
    $result = $query->condition($or)->accessCheck(FALSE)->execute();
    $this->assertEquals(
      [
        1002 => 1002,
        1003 => 1003,
        1006 => 1006,
      ],
      $result,
      'query with field_alpha ending by ta or starting with te'
    );

    // Query with field_alpha ending by "ta" or starting with "te" and
    // field_beta != 64.
    $query = $this->entity_type_manager->getStorage('simple_external_entity')->getQuery();
    $or = $query->orConditionGroup()->condition('field_alpha', 'ta', 'ENDS_WITH')->condition('field_alpha', 'te', 'STARTS_WITH');
    $result = $query->condition('field_beta', 64, '<>')->condition($or)->accessCheck(FALSE)->execute();
    $this->assertEquals(
      [
        1002 => 1002,
        1003 => 1003,
      ],
      $result,
      'query with field_alpha ending by ta or starting with te and field_beta != 64'
    );

    // Complex query 1.
    $query = $this->entity_type_manager->getStorage('simple_external_entity')->getQuery();
    $and1 = $query->andConditionGroup()->condition('field_beta', [16, 42, 51], 'IN')->condition('field_alpha', 'ti', 'CONTAINS');
    $and2 = $query->andConditionGroup()->condition('field_beta', [30, 50], 'BETWEEN')->condition('field_gama', 'xyz');
    $or = $query->orConditionGroup()->condition($and1)->condition($and2);
    $result = $query->condition('field_alpha', 'to', 'STARTS_WITH')->condition($or)->accessCheck(FALSE)->execute();
    $this->assertEquals(
      [
        1008 => 1008,
        1009 => 1009,
      ],
      $result,
      'complex query 1'
    );
  }

}
